use strict;

package Flickr::Upload::Dopplr;
use base qw (Flickr::Upload);

$Flickr::Upload::Dopplr::VERSION = '0.02';

# $Id: Dopplr.pm,v 1.3 2007/09/09 18:30:06 asc Exp $

=head1 NAME

Flickr::Upload::Dopplr - Flickr::Upload subclass to assign location information using Dopplr

=head1 SYNOPSIS

 use Flickr::Upload::Dopplr;

 my $flickr_apikey = "OH HAI";
 my $flickr_apisecret = "OH NOES";
 my $flickr_authtoken = "O RLY";
 my $dopplr_authtoken = "JONEZ";

 my %args = ('key' => $flickr_apikey,
             'secret' => $flickr_apisecret,
             'dopplr_token' => $dopplr_authtoken);

 my $uploadr = Flickr::Upload::Dopplr->new(\%args);

 my $photo_id = $uploadr->upload('photo' => "/path/to/photo",
                                 'auth_token' => $flickr_authtoken);


=head1 DESCRIPTION

Flickr::Upload subclass to assign location information using Dopplr.

Specifically, the package will query Dopplr for the current location of the
user associated with I<$dopplr_authtoken> and assign the following tags for
the name of the city a machinetag representing the Geonames.org ID for that
city.

If, when the photo is uploaded, the Dopplr API thinks that it is a "travel
day" another machine tag (dopplr:trip=) will be added containing the numeric
identifier for that trip.

At the moment there is no way to modify (lower case, normalize, etc.) any of
the tag strings.

If an upload is successful, the package will attempt to assign latitude and
longitude information for the photo with a Flickr accuracy of 11 (or "city").

At the moment, the package does not check to see whether geo information was
already assigned (for example, via GPS EXIF data) nor does it issue and error
reporting if there was a problem assigning geo information.

=cut

use Net::Dopplr;

=head1 PACKAGE METHODS

=head2 __PACKAGE__->new(\%args)

All the same arguments required by the I<Flickr::Upload> constructor plus the
following :

=over 4

=item * B<dopplr_token>

String. I<required>

A valid Dopplr API authentication token.

=back

Returns a I<Flickr::Upload::Dopplr> object.

=cut

sub new {
        my $pkg = shift;
        my $args = shift;

        my $token = $args->{'dopplr_token'};
        delete($args->{'dopplr_token'});

        my $self = $pkg->SUPER::new($args);

        my $dopplr = Net::Dopplr->new($token);
        $self->{'__dopplr'} = $dopplr;

        return $self;
}

=head1 OBJECT METHODS YOU SHOULD CARE ABOUT

=head2 $obj->upload(%args)

Nothing you wouldn't pass the Flickr::Upload I<upload> method.

At the moment, there is no useful error reporting. This method will either
return a photo ID, or not, independent of whether any information retrieved
from or set using the Dopplr API was successful.

=cut

sub upload {
        my $self = shift;
        my %args = @_;

        my $city = $self->whereami();
                
        if (! $city){
                return $self->SUPER::upload(%args);
        }

        $args{'tags'} .= sprintf(" \"%s\"", $city->{'name'});
        $args{'tags'} .= sprintf(" geonames:locality=%d", $city->{'geoname_id'});

        if ($city->{'tripid'}){
                $args{'tags'} .= sprintf(" dopplr:trip=%d", $city->{'tripid'});
        }

        my $photo_id = $self->SUPER::upload(%args);

        if (! $photo_id){
                return 0;
        }

        my %geo = ('accuracy' => 11,
                   'lat' => $city->{'latitude'},
                   'lon' => $city->{'longitude'},
                   'auth_token' => $args{'auth_token'},
                   'photo_id' => $photo_id);

        my $res = $self->execute_method('flickr.photos.geo.setLocation', \%geo);
        return $photo_id;
}

=head1 OBJECT METHODS YOU MAY CARE ABOUT

=head2 $obj->whereami()

Returns a hash containing I<current_city> information as defined by
the Dopplr API.

This hash is used to assign tags and latitude/longitude information.

=cut

sub whereami {
        my $self = shift;

        my $info = $self->{'__dopplr'}->traveller_info();

        if (! $info){
                return undef;
        }
        
        my $city = $info->{'traveller'}->{'current_city'};

        if ($info->{'traveller'}->{'travel_today'}){
                $city->{'tripid'} = $info->{'traveller'}->{'current_trip'}->{'id'};
        }

        return $city;
}

=head1 VERSION

0.02

=head1 DATE

$Date: 2007/09/09 18:30:06 $

=head1 AUTHOR

Aaron Straup Cope <ascope@cpan.org>

=head1 SEE ALSO

I<Flickr::Upload>

I<Net::Dopplr>

=head1 BUGS

Please report all bugs via http://rt.cpan.org/

=head1 LICENSE

Copyright (c) 2007 Aaron Straup Cope. All Rights Reserved.

This is free software. You may redistribute it and/or
modify it under the same terms as Perl itself.

=cut

return 1;
